"""
Module simplifying manipulation of XML described at
http://libvirt.org/formatnwfilter.html
"""

from virttest.libvirt_xml import base, accessors
from virttest.libvirt_xml.devices.filterref import Filterref


class NwfilterBinding(base.LibvirtXMLBase):
    """
    Accessor methods for NwfilterXML class
    simple example:

    <filterbinding>
    <owner>
    <name>rhel</name>
    <uuid>c9543ff4-1703-40a9-abbf-7f3e0fcd66c0</uuid>
    </owner>
    <portdev name='vnet0'/>
    <mac address='52:54:00:fa:81:87'/>
    <filterref filter='clean-traffic'>
    <parameter name='MAC' value='52:54:00:fa:81:87'/>
    </filterref>
    </filterbinding>


    Properties:
        owner:
        uuid: string,  domain uuid of vm
        portdev: port device of host
        mac_address: mac of the vnet0
        filterref:  list, list of dictionaries describing filterref properties
    """

    __slots__ = base.LibvirtXMLBase.__slots__ + ('owner', 'portdev',
                                                 'mac_address', 'filterref',)

    def __init__(self, virsh_instance=base.virsh):
        accessors.XMLElementNest("owner", self,
                                 parent_xpath='/',
                                 tag_name='owner',
                                 subclass=self.Owner,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLAttribute('portdev', self,
                               parent_xpath='/',
                               tag_name='portdev',
                               attribute='name')
        accessors.XMLAttribute('mac_address', self,
                               parent_xpath='/',
                               tag_name='mac',
                               attribute='address')
        accessors.XMLElementNest("filterref", self,
                                 parent_xpath='/',
                                 tag_name='filterref',
                                 subclass=Filterref,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        super(NwfilterBinding, self).__init__(virsh_instance=virsh_instance)

        self.xml = u'<filterbinding></filterbinding>'

    def new_filterref(self, **dargs):
        """
        Return a new interafce filterref instance from dargs
        """
        new_one = Filterref(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def new_owner(self, name, uuid):
        """
        Return a new interafce Owner instance from name and uuid
        """
        new_one = self.Owner(virsh_instance=self.virsh)
        new_one.name = name
        new_one.uuid = uuid
        return new_one

    class Owner(base.LibvirtXMLBase):
        """
        Interface Owner xml class

        Properties:
        name: string, name of the vm
        uuid: string, domain uuid of vm
        """
        __slots__ = ("name", "uuid")

        def __init__(self, virsh_instance=base.virsh):
            accessors.XMLElementText('name', self,
                                     parent_xpath='/',
                                     tag_name='name')
            accessors.XMLElementText('uuid', self,
                                     parent_xpath='/',
                                     tag_name='uuid')
            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = u'<owner></owner>'



import logging
import os
import re
import glob
import shutil
from six import StringIO
try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser

from avocado.utils import astring
from avocado.utils import process
from avocado.utils import genio
from avocado.utils import crypto
from avocado.utils import download
from avocado.utils import git

from six.moves import urllib
from six import string_types

from virttest import data_dir


class ConfigLoader:

    """
    Base class of the configuration parser
    """

    def __init__(self, cfg, tmpdir=data_dir.get_tmp_dir(), raise_errors=False):
        """
        Instantiate configparser and load data.

        :param cfg: Where we'll get configuration data. It can be either:
                * A URL containing the file
                * A valid file path inside the filesystem
                * A string containing configuration data
        :param tmpdir: Where we'll dump the temporary conf files.
        :param raise_errors: Whether config value absences will raise
                ValueError exceptions.
        """
        # Base Parser
        self.parser = ConfigParser.ConfigParser()
        # Raise errors when lacking values
        self.raise_errors = raise_errors
        # File is already a file like object
        if hasattr(cfg, 'read'):
            self.cfg = cfg
            self.parser.readfp(self.cfg)
        elif isinstance(cfg, string_types):
            # Config file is a URL. Download it to a temp dir
            if cfg.startswith('http') or cfg.startswith('ftp'):
                self.cfg = os.path.join(tmpdir, os.path.basename(cfg))
                download.url_download(cfg, self.cfg)
                self.parser.read(self.cfg)
            # Config is a valid filesystem path to a file.
            elif os.path.exists(os.path.abspath(cfg)):
                if os.path.isfile(cfg):
                    self.cfg = os.path.abspath(cfg)
                    self.parser.read(self.cfg)
                else:
                    e_msg = 'Invalid config file path: %s' % cfg
                    raise IOError(e_msg)
            # Config file is just a string, convert it to a python file like
            # object using StringIO
            else:
                self.cfg = StringIO(cfg)
                self.parser.readfp(self.cfg)

    def get(self, section, option, default=None):
        """
        Get the value of a option.

        Section of the config file and the option name.
        You can pass a default value if the option doesn't exist.

        :param section: Configuration file section.
        :param option: Option we're looking after.
        :default: In case the option is not available and raise_errors is set
                to False, return the default.
        """
        if not self.parser.has_option(section, option):
            if self.raise_errors:
                raise ValueError('No value for option %s. Please check your '
                                 'config file "%s".' % (option, self.cfg))
            else:
                return default

        return self.parser.get(section, option)

    def set(self, section, option, value):
        """
        Set an option.

        This change is not persistent unless saved with 'save()'.
        """
        if not self.parser.has_section(section):
            self.parser.add_section(section)
        return self.parser.set(section, option, value)

    def remove(self, section, option):
        """
        Remove an option.
        """
        if self.parser.has_section(section):
            self.parser.remove_option(section, option)

    def save(self):
        """
        Save the configuration file with all modifications
        """
        if not self.cfg:
            return
        with open(self.cfg, 'w') as fileobj:
            self.parser.write(fileobj)

    def check(self, section):
        """
        Check if the config file has valid values
        """
        if not self.parser.has_section(section):
            return False, "Section not found: %s" % (section)

        options = self.parser.items(section)
        for i in range(options.__len__()):
            param = options[i][0]
            aux = str.split(param, '.')

            if aux.__len__ < 2:
                return False, "Invalid parameter syntax at %s" % (param)

            if not self.check_parameter(aux[0], options[i][1]):
                return False, "Invalid value at %s" % (param)

        return True, None

    def check_parameter(self, param_type, parameter):
        """
        Check if a option has a valid value
        """
        if parameter == '' or parameter is None:
            return False
        elif param_type == "ip" and self.__isipaddress(parameter):
            return True
        elif param_type == "int" and self.__isint(parameter):
            return True
        elif param_type == "float" and self.__isfloat(parameter):
            return True
        elif param_type == "str" and self.__isstr(parameter):
            return True

        return False

    def __isipaddress(self, parameter):
        """
        Verify if the ip address is valid

        :param ip String: IP Address
        :return: True if a valid IP Address or False
        """
        octet1 = "([1-9][0-9]{,1}|1[0-9]{2}|2[0-4][0-9]|25[0-5])"
        octet = "([0-9]{1,2}|1[0-9]{2}|2[0-4][0-9]|25[0-5])"
        pattern = "^" + octet1 + "\.(" + octet + "\.){2}" + octet + "$"
        if re.match(pattern, parameter) is None:
            return False
        else:
            return True

    def __isint(self, parameter):
        try:
            int(parameter)
        except Exception as e_stack:
            return False
        return True

    def __isfloat(self, parameter):
        try:
            float(parameter)
        except Exception as e_stack:
            return False
        return True

    def __isstr(self, parameter):
        try:
            str(parameter)
        except Exception as e_stack:
            return False
        return True


def get_known_backends():
    """
    Return virtualization backends supported by avocado-vt.
    """
    # Generic means the test can run in multiple backends, such as libvirt
    # and qemu.
    known_backends = ['generic', 'multi_host_migration']
    known_backends += os.listdir(data_dir.BASE_BACKEND_DIR)
    return known_backends


def get_test_provider_names(backend=None):
    """
    Get the names of all test providers available in test-providers.d.

    :return: List with the names of all test providers.
    """
    provider_name_list = []
    provider_dir = data_dir.get_test_providers_dir()
    for provider in glob.glob(os.path.join(provider_dir, '*.ini')):
        provider_name = os.path.basename(provider).split('.')[0]
        provider_info = get_test_provider_info(provider_name)
        if backend is not None:
            if backend in provider_info['backends']:
                provider_name_list.append(provider_name)
        else:
            provider_name_list.append(provider_name)
    return provider_name_list


def get_test_provider_subdirs(backend=None):
    """
    Get information of all test provider subdirs for a given backend.

    If no backend is provided, return all subdirs with tests.

    :param backend: Backend type, such as 'qemu'.
    :return: List of directories that contain tests for the given backend.
    """
    subdir_list = []
    for provider_name in get_test_provider_names():
        provider_info = get_test_provider_info(provider_name)
        backends_info = provider_info['backends']
        if backend is not None:
            if backend in backends_info:
                subdir_list.append(backends_info[backend]['path'])
        else:
            for b in backends_info:
                subdir_list.append(backends_info[b]['path'])
    return subdir_list


def get_test_provider_info(provider):
    """
    Get a dictionary with relevant test provider info, such as:

    * provider uri (git repo or filesystem location)
    * provider git repo data, such as branch, ref, pubkey
    * backends that this provider has tests for. For each backend type the
        provider has tests for, the 'path' will be also available.

    :param provider: Test provider name, such as 'io-github-autotest-qemu'.
    """
    provider_info = {}
    provider_path = os.path.join(data_dir.get_test_providers_dir(),
                                 '%s.ini' % provider)
    provider_cfg = ConfigLoader(provider_path)
    provider_info['name'] = provider
    provider_info['uri'] = provider_cfg.get('provider', 'uri')
    provider_info['branch'] = provider_cfg.get('provider', 'branch', 'master')
    provider_info['ref'] = provider_cfg.get('provider', 'ref')
    provider_info['pubkey'] = provider_cfg.get('provider', 'pubkey')
    provider_info['backends'] = {}

    for backend in get_known_backends():
        subdir = provider_cfg.get(backend, 'subdir')
        cart_cfgs = provider_cfg.get(backend, 'configs')

        backend_dic = {}
        if cart_cfgs is not None:
            # Give ability to specify few required configs separated with comma
            cart_cfgs = [x.strip() for x in cart_cfgs.split(',')]
            backend_dic.update({'cartesian_configs': cart_cfgs})

        if subdir is not None:
            if provider_info['uri'].startswith('file://'):
                src = os.path.join(provider_info['uri'][7:],
                                   subdir)
            else:
                src = os.path.join(data_dir.get_test_provider_dir(provider),
                                   subdir)
            backend_dic.update({'path': src})
            provider_info['backends'].update({backend: backend_dic})

    return provider_info


def download_test_provider(provider, update=False):
    """
    Download a test provider defined on a .ini file inside test-providers.d.

    This function will only download test providers that are in git repos.
    Local filesystems don't need this functionality.

    :param provider: Test provider name, such as 'io-github-autotest-qemu'.
    """
    provider_info = get_test_provider_info(provider)
    uri = provider_info.get('uri')
    if not uri.startswith('file://'):
        uri = provider_info.get('uri')
        branch = provider_info.get('branch')
        ref = provider_info.get('ref')
        pubkey = provider_info.get('pubkey')
        download_dst = data_dir.get_test_provider_dir(provider)
        dir_existed = os.path.exists(download_dst)
        repo_downloaded = os.path.isdir(os.path.join(download_dst, '.git'))
        original_dir = os.getcwd()
        try:
            if not repo_downloaded or update:
                download_dst = git.get_repo(uri=uri, branch=branch, commit=ref,
                                            destination_dir=download_dst)
                os.chdir(download_dst)
                try:
                    process.run('git remote add origin %s' % uri)
                except process.CmdError:
                    pass
        except Exception:
            if not dir_existed and os.path.isdir(download_dst):
                logging.error('Cleaning up provider %s download dir %s', provider,
                              download_dst)
                shutil.rmtree(download_dst)
            raise

        # sanity check to ensure the git repository is OK:
        try:
            os.chdir(download_dst)
            process.system('git log -1')
        except process.CmdError:
            logging.error('Something is unexpectedly wrong with the git repository at %s',
                          download_dst)
            raise
        finally:
            os.chdir(original_dir)


def test_providers_not_downloaded():
    """
    Return the list of test providers that have not being downloaded
    """
    result = []
    for provider in get_test_provider_names():
        download_dst = data_dir.get_test_provider_dir(provider)
        repo_downloaded = os.path.isdir(os.path.join(download_dst, '.git'))
        if not repo_downloaded:
            result.append(provider)
    return result


def download_all_test_providers(update=False):
    """
    Download all available test providers.
    """
    for provider in get_test_provider_names():
        download_test_provider(provider, update)


def get_all_assets():
    asset_data_list = []
    download_dir = data_dir.get_download_dir()
    for asset in glob.glob(os.path.join(download_dir, '*.ini')):
        asset_name = os.path.basename(asset)[:-4]
        asset_data_list.append(get_asset_info(asset_name))
    return asset_data_list


def get_file_asset(title, src_path, destination):
    if not os.path.isabs(destination):
        destination = os.path.join(data_dir.get_data_dir(), destination)

    for ext in (".xz", ".gz", ".7z", ".bz2"):
        if os.path.exists(src_path + ext):
            destination = destination + ext
            logging.debug('Found source image %s', destination)
            return {
                'url': None, 'sha1_url': None, 'destination': src_path + ext,
                'destination_uncompressed': destination,
                'uncompress_cmd': None, 'shortname': title, 'title': title,
                'downloaded': True}

    if os.path.exists(src_path):
        logging.debug('Found source image %s', destination)
        return {'url': src_path, 'sha1_url': None, 'destination': destination,
                'destination_uncompressed': None, 'uncompress_cmd': None,
                'shortname': title, 'title': title,
                'downloaded': os.path.exists(destination)}

    return None


def get_asset_info(asset, ini_dir=None, section=None):
    """"
    Parse $asset.ini file and returns a dictionary suitable for
    asset.download_file()

    :param asset: Asset name. A file ending in .ini.
    :param ini_dir: Directory where to search .ini file.
    :param section: Name of the [] section in .ini file. If None, then use
                    asset name.
    """
    asset_info = {}
    ini_dir = ini_dir or data_dir.get_download_dir()
    asset_path = os.path.join(ini_dir, '%s.ini' % asset)
    assert os.path.exists(asset_path), "Missing asset file %s" % asset_path
    asset_cfg = ConfigLoader(asset_path)

    section = section or asset
    asset_info['url'] = asset_cfg.get(section, 'url')
    asset_info['sha1_url'] = asset_cfg.get(section, 'sha1_url')
    asset_info['title'] = asset_cfg.get(section, 'title')
    destination = asset_cfg.get(section, 'destination')
    if not os.path.isabs(destination):
        destination = os.path.join(data_dir.get_data_dir(), destination)
    asset_info['destination'] = destination
    asset_info['asset_exists'] = os.path.isfile(destination)

    # Optional fields
    d_uncompressed = asset_cfg.get(section, 'destination_uncompressed')
    if d_uncompressed is not None and not os.path.isabs(d_uncompressed):
        d_uncompressed = os.path.join(data_dir.get_data_dir(),
                                      d_uncompressed)
    asset_info['destination_uncompressed'] = d_uncompressed
    asset_info['uncompress_cmd'] = asset_cfg.get(section, 'uncompress_cmd')

    return asset_info


def uncompress_asset(asset_info, force=False):
    destination = asset_info['destination']
    uncompress_cmd = asset_info['uncompress_cmd']
    destination_uncompressed = asset_info['destination_uncompressed']

    archive_re = re.compile(r".*\.(gz|xz|7z|bz2)$")
    if destination_uncompressed is not None:
        if uncompress_cmd is None:
            match = archive_re.match(destination)
            if match:
                if match.group(1) == 'gz':
                    uncompress_cmd = ('gzip -cd %s > %s' %
                                      (destination, destination_uncompressed))
                elif match.group(1) == 'xz':
                    uncompress_cmd = ('xz -cd %s > %s' %
                                      (destination, destination_uncompressed))
                elif match.group(1) == 'bz2':
                    uncompress_cmd = ('bzip2 -cd %s > %s' %
                                      (destination, destination_uncompressed))
                elif match.group(1) == '7z':
                    uncompress_cmd = '7za -y e %s' % destination
        else:
            uncompress_cmd = "%s %s" % (uncompress_cmd, destination)

    if uncompress_cmd is not None:
        uncompressed_file_exists = os.path.exists(destination_uncompressed)
        force = (force or not uncompressed_file_exists)

        if os.path.isfile(destination) and force:
            os.chdir(os.path.dirname(destination_uncompressed))
            logging.debug('Uncompressing %s -> %s', destination,
                          destination_uncompressed)
            process.run(uncompress_cmd, shell=True)
            backup_file = destination_uncompressed + '.backup'
            if os.path.isfile(backup_file):
                logging.debug('Copying %s -> %s', destination_uncompressed,
                              backup_file)
                shutil.copy(destination_uncompressed, backup_file)


def download_file(asset_info, interactive=False, force=False):
    """
    Verifies if file that can be find on url is on destination with right hash.

    This function will verify the SHA1 hash of the file. If the file
    appears to be missing or corrupted, let the user know.

    :param asset_info: Dictionary returned by get_asset_info
    """
    file_ok = False
    problems_ignored = False
    had_to_download = False
    sha1 = None

    url = asset_info['url']
    sha1_url = asset_info['sha1_url']
    destination = asset_info['destination']
    title = asset_info['title']

    if sha1_url is not None:
        try:
            logging.info("Verifying expected SHA1 sum from %s", sha1_url)
            sha1_file = urllib.request.urlopen(sha1_url)
            sha1_contents = astring.to_text(sha1_file.read())
            sha1 = sha1_contents.split(" ")[0]
            logging.info("Expected SHA1 sum: %s", sha1)
        except Exception as e:
            logging.error("Failed to get SHA1 from file: %s", e)
    else:
        sha1 = None

    destination_dir = os.path.dirname(destination)
    if not os.path.isdir(destination_dir):
        os.makedirs(destination_dir)

    if not os.path.isfile(destination):
        logging.warning("File %s not found", destination)
        if interactive:
            answer = genio.ask("Would you like to download it from %s?" % url)
        else:
            answer = 'y'
        if answer == 'y':
            try:
                download.url_download_interactive(url, destination,
                                                  "Downloading %s" % title)
                had_to_download = True
            except Exception as download_failure:
                logging.error("Check your internet connection: %s",
                              download_failure)
        else:
            logging.warning("Missing file %s", destination)
    else:
        logging.info("Found %s", destination)
        if sha1 is None:
            answer = 'n'
        else:
            answer = 'y'

        if answer == 'y':
            actual_sha1 = crypto.hash_file(destination, algorithm='sha1')
            if actual_sha1 != sha1:
                logging.info("Actual SHA1 sum: %s", actual_sha1)
                if interactive:
                    answer = genio.ask("The file seems corrupted or outdated. "
                                       "Would you like to download it?")
                else:
                    logging.info("The file seems corrupted or outdated")
                    answer = 'y'
                if answer == 'y':
                    logging.info("Updating image to the latest available...")
                    while not file_ok:
                        try:
                            download.url_download_interactive(url, destination,
                                                              title)
                        except Exception as download_failure:
                            logging.error("Check your internet connection: %s",
                                          download_failure)
                        sha1_post_download = crypto.hash_file(destination,
                                                              algorithm='sha1')
                        had_to_download = True
                        if sha1_post_download != sha1:
                            logging.error("Actual SHA1 sum: %s", actual_sha1)
                            if interactive:
                                answer = genio.ask("The file downloaded %s is "
                                                   "corrupted. Would you like "
                                                   "to try again?" %
                                                   destination)
                            else:
                                answer = 'n'
                            if answer == 'n':
                                problems_ignored = True
                                logging.error("File %s is corrupted" %
                                              destination)
                                file_ok = True
                            else:
                                file_ok = False
                        else:
                            file_ok = True
            else:
                file_ok = True
                logging.info("SHA1 sum check OK")
        else:
            problems_ignored = True
            logging.info("File %s present, but did not verify integrity",
                         destination)

    if file_ok:
        if not problems_ignored:
            logging.info("%s present, with proper checksum", destination)

    uncompress_asset(asset_info=asset_info, force=force or had_to_download)


def download_asset(asset, interactive=True, restore_image=False):
    """
    Download an asset defined on an asset file.

    Asset files are located under /shared/downloads, are .ini files with the
    following keys defined:

    title
        Title string to display in the download progress bar.
    url
        URL of the resource
    sha1_url
        URL with SHA1 information for the resource, in the form
        sha1sum file_basename
    destination
        Location of your file relative to the data directory
        (TEST_SUITE_ROOT/shared/data)
    destination
        Location of the uncompressed file relative to the data
        directory (TEST_SUITE_ROOT/shared/data)
    uncompress_cmd
        Command that needs to be executed with the compressed
        file as a parameter

    :param asset: String describing an asset file.
    :param interactive: Whether to ask the user before downloading the file.
    :param restore_image: If the asset is a compressed image, we can uncompress
                          in order to restore the image.
    """
    asset_info = get_asset_info(asset)

    download_file(asset_info=asset_info, interactive=interactive,
                  force=restore_image)



"""
IP sniffing facilities
"""

import threading
import logging
import re
from collections import Iterable

import aexpect
from aexpect.remote import handle_prompts
from avocado.utils import path as utils_path

import six

from virttest.utils_misc import log_line


class AddrCache(object):

    """
    Address cache implementation.
    """

    def __init__(self):
        """Initializes the address cache."""
        self._data = {}
        self._lock = threading.RLock()

    def __repr__(self):
        return repr(self._data)

    def __getstate__(self):
        state = self.__dict__.copy()
        del state["_lock"]
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)
        self._lock = threading.RLock()

    @staticmethod
    def _format_hwaddr(hwaddr):
        return hwaddr.lower()

    def __setitem__(self, hwaddr, ipaddr):
        hwaddr = self._format_hwaddr(hwaddr)
        with self._lock:
            if self._data.get(hwaddr) == ipaddr:
                return
            self._data[hwaddr] = ipaddr
        logging.debug("Updated HWADDR (%s)<->(%s) IP pair "
                      "into address cache", hwaddr, ipaddr)

    def __getitem__(self, hwaddr):
        hwaddr = self._format_hwaddr(hwaddr)
        ipaddr = None
        with self._lock:
            ipaddr = self._data.get(hwaddr)
        return ipaddr

    def __delitem__(self, hwaddr):
        hwaddr = self._format_hwaddr(hwaddr)
        with self._lock:
            if hwaddr not in self._data:
                return
            del self._data[hwaddr]
        logging.debug("Dropped the address cache of HWADDR (%s)", hwaddr)

    def get(self, hwaddr):
        """
        Get the ip address of the given hardware address.

        :param hwaddr: Hardware address.
        """
        return self.__getitem__(hwaddr)

    def drop(self, hwaddr):
        """
        Drop the cache of the given hardware address.

        :param hwaddr: Hardware address.
        """
        return self.__delitem__(hwaddr)

    def update(self, cache):
        """
        Update the address cache with the address pairs from other,
        overwriting existing hardware address.

        :param cache: AddrCache object, dict object or iterable
                      key/value pairs.
        """
        if self == cache:
            return
        if isinstance(cache, AddrCache):
            _cache = cache
            with _cache._lock:
                cache = _cache._data.copy()
        if isinstance(cache, Iterable):
            cache = six.iteritems(cache)
        for hwaddr, ipaddr in cache:
            self[hwaddr] = ipaddr

    def clear(self):
        """Clear all the address caches."""
        with self._lock:
            self._data.clear()
        logging.debug("Clean out all the address caches")


class Sniffer(object):

    """
    Virtual base class of the ip sniffer abstraction.

    The `_output_handler(self, line)` method must be
    provided by the subclasses.
    """

    #: Sniffer's command name
    command = ""
    #: Sniffer's options
    options = ""
    # Regexp to find "process terminated" line in sniffer output
    _re_sniffer_finished = re.compile(r"\(Process terminated with status (\d+)")

    def __init__(self, addr_cache, log_file, remote_opts=None):
        """
        Initializes the sniffer.

        :param addr_cache: Address cache to be updated.
        :param log_file: Log file name.
        :param remote_opts: Options of remote host session, is a tuple built
                            up by `(address, port, username, password,
                            prompt)`. If provided, launches the sniffer on
                            remote host, otherwise on local host.
        """
        self._cache = addr_cache
        self._logfile = log_file
        self._remote_opts = remote_opts
        self._process = None

    @classmethod
    def _is_supported_remote(cls, session):
        return session.cmd_status("which %s" % cls.command) == 0

    @classmethod
    def is_supported(cls, session=None):
        """
        Check if host supports the sniffer.

        :param session: Remote host session. If provided, performs the
                        check on remote host, otherwise on local host.
        """
        if session:
            return cls._is_supported_remote(session)
        try:
            utils_path.find_command(cls.command)
            return True
        except utils_path.CmdNotFoundError:
            return False

    def _output_handler(self, line):
        raise NotImplementedError

    def _output_logger_handler(self, line):
        try:
            log_line(self._logfile, line)
        except Exception as e:
            logging.warn("Can't log ip sniffer output: '%s'", e)
        if self._output_handler(line):
            return
        # We can check whether the process is terminated unexpectedly
        # here since the terminated status is a line of the output
        match = self._re_sniffer_finished.match(line)
        if match:
            if match.group(1) != "0":
                logging.error("IP sniffer (%s) terminated unexpectedly! "
                              "please check the log to get the details "
                              "(status: %s)", self.command, match.group(1))

    def _start_remote(self):
        address, port, username, password, prompt = self._remote_opts
        cmd = "%s %s" % (self.command, self.options)
        logging.debug("Run '%s' on host '%s'", cmd, address)
        login_cmd = ("ssh -o UserKnownHostsFile=/dev/null "
                     "-o StrictHostKeyChecking=no "
                     "-o PreferredAuthentications=password -p %s %s@%s" %
                     (port, username, address))

        self._process = aexpect.ShellSession(
            login_cmd,
            output_func=self._output_logger_handler)
        handle_prompts(self._process, username, password, prompt)
        self._process.sendline(cmd)

    def _start(self):
        cmd = "%s %s" % (self.command, self.options)
        if self._remote_opts:
            self._start_remote()
        else:
            self._process = aexpect.run_tail(
                command=cmd,
                output_func=self._output_logger_handler)

    def is_alive(self):
        """Check if the sniffer is alive."""
        if not self._process:
            return False
        return self._process.is_alive()

    def start(self):
        """Start sniffing."""
        if self.is_alive():
            return
        self.stop()
        self._start()

    def stop(self):
        """Stop sniffing."""
        if self._process:
            self._process.close()
            self._process = None

    def __del__(self):
        self.stop()


class TcpdumpSniffer(Sniffer):

    """
    Tcpdump sniffer class.
    """

    command = "tcpdump"
    options = "-tnpvvvi any 'port 68 or port 546'"

    def __init__(self, addr_cache, log_file, remote_opts=None):
        super(TcpdumpSniffer, self).__init__(addr_cache, log_file, remote_opts)
        self._context = {}

    def _output_handler(self, line):
        # BootP/DHCP (RFC 951/2131)
        matches = re.search(r"^IP\s", line)
        if matches:
            # Clear context upon receiving new packets
            self._context.clear()
            return True

        matches = re.search(r"Your.IP\s+(\S+)", line, re.I)
        if matches:
            self._context["ip"] = matches.group(1)
            return True

        matches = re.search(r"Client.Ethernet.Address\s+(\S+)", line, re.I)
        if matches:
            self._context["mac"] = matches.group(1)
            return True

        if re.search(r"DHCP.Message.*:\s+ACK", line, re.I):
            mac = self._context.get("mac")
            ip = self._context.get("ip")
            if mac and ip:
                self._cache[mac] = ip
            return True

        # DHCPv6 (RFC 3315)
        if re.search("dhcp6 reply", line, re.I):
            regex = "IA_ADDR (.*) pltime.*client-ID.*?([0-9a-fA-F]{12})\)"
            matches = re.search(regex, line, re.I)
            if matches:
                info = matches.groups()
                mac = ":".join(re.findall("..", info[1]))
                ip = info[0].lower()
                self._cache["%s_6" % mac] = ip
            return True


class TSharkSniffer(Sniffer):

    """
    TShark sniffer class.
    """

    command = "tshark"
    options = ("-npi any -T fields -E separator=/s -E occurrence=f "
               "-E header=y -e ip.src -e ip.dst -e bootp.type -e bootp.id "
               "-e bootp.hw.mac_addr -e bootp.ip.your -e bootp.option.dhcp "
               "-e ipv6.src -e ipv6.dst "
               # Positional arguments must be the last arguments
               "'port 68 or port 546'")

    def _output_handler(self, line):
        packet = line.lstrip().split()
        if not len(packet):
            return True

        # BootP/DHCP (RFC 951/2131)
        if re.match(r"\d+\.\d+\.\d+\.\d+", packet[0]):
            try:
                chaddr = packet[4]
                yiaddr = packet[5]
                m_type = packet[6]
            except IndexError:
                # Ignore problematical packets
                return True
            if m_type == "5" and yiaddr != "0.0.0.0":
                # Update cache only if get the ACK reply
                # and the previous request is not INFORM
                self._cache[chaddr] = yiaddr
            return True

        # DHCPv6 (RFC 3315)
        if re.match(r"[0-9a-fA-F]{1,4}:\S+", packet[0]):
            # TODO: support DHCPv6
            if not self.__dict__.setdefault("_ip6_warned", False):
                logging.warn("IPv6 address sniffing is not supported yet by "
                             "using TShark, please fallback to use other "
                             "sniffers by uninstalling TShark when testing "
                             "with IPv6")
                self._ip6_warned = True
            return True


#: All the defined sniffers
Sniffers = (TSharkSniffer, TcpdumpSniffer)



from avocado.core import plugin_interfaces
from avocado.core.settings import settings
from avocado.utils import path as utils_path
from virttest.compat import is_registering_settings_required
from virttest.defaults import DEFAULT_MACHINE_TYPE
from virttest.standalone_test import (SUPPORTED_DISK_BUSES,
                                      SUPPORTED_IMAGE_TYPES,
                                      SUPPORTED_NIC_MODELS,
                                      find_default_qemu_paths)

if hasattr(plugin_interfaces, 'Init'):
    class VtInit(plugin_interfaces.Init):

        name = 'vt-init'
        description = "VT plugin initilization"

        def initialize(self):
            if not is_registering_settings_required():
                return

            # [vt.setup] section
            section = 'vt.setup'
            help_msg = 'Backup image before testing (if not already backed up)'
            settings.register_option(section, 'backup_image_before_test',
                                     help_msg=help_msg, key_type=bool,
                                     default=True)

            help_msg = 'Restore image after testing (if backup present)'
            settings.register_option(section, 'restore_image_after_test',
                                     help_msg=help_msg, key_type=bool,
                                     default=True)

            help_msg = 'Keep guest running between tests (faster, but unsafe)'
            settings.register_option(section, 'keep_guest_running',
                                     help_msg=help_msg, key_type=bool,
                                     default=False)

            # [vt.common] section
            section = 'vt.common'

            help_msg = ('Data dir path. If none specified, the default '
                        'virt-test data dir will be used')
            settings.register_option(section, 'data_dir',
                                     help_msg=help_msg,
                                     default='')

            help_msg = ('Make the temporary dir path persistent across jobs if'
                        ' needed. By default the data in the temporary '
                        'directory will be wiped after each test in some cases'
                        ' and after each job in others.')
            settings.register_option(section, 'tmp_dir',
                                     help_msg=help_msg,
                                     default='')

            help_msg = ('Enable only type specific tests. Shared tests will '
                        'not be tested')
            settings.register_option(section, 'type_specific_only',
                                     help_msg=help_msg, key_type=bool,
                                     default=False)

            help_msg = ('RAM dedicated to the main VM. Usually defaults to '
                        '1024, as set in "base.cfg", but can be a different '
                        'value depending on the various other configuration '
                        'files such as configuration files under "guest-os" '
                        'and test provider specific files')
            settings.register_option(section, 'mem',
                                     help_msg=help_msg,
                                     default=None)

            help_msg = 'Architecture under test'
            settings.register_option(section, 'arch',
                                     help_msg=help_msg,
                                     default=None)

            help_msg = 'Machine type under test'
            settings.register_option(section, 'machine_type',
                                     help_msg=help_msg,
                                     default=DEFAULT_MACHINE_TYPE)

            help_msg = 'Nettype (bridge, user, none)'
            settings.register_option(section, 'nettype',
                                     help_msg=help_msg,
                                     default='')

            help_msg = 'Bridge name to be used if you select bridge as a nettype'
            settings.register_option(section, 'netdst',
                                     help_msg=help_msg,
                                     default='virbr0')

            # [vt.qemu] section
            section = 'vt.qemu'

            try:
                default_qemu_bin_path = find_default_qemu_paths()[0]
            except (RuntimeError, utils_path.CmdNotFoundError):
                default_qemu_bin_path = None
            help_msg = 'Path to a custom qemu binary to be tested'
            settings.register_option(section, 'qemu_bin',
                                     help_msg=help_msg,
                                     default=default_qemu_bin_path)

            help_msg = ('Path to a custom qemu binary to be tested for the '
                        'destination of a migration, overrides qemu_bin for '
                        'that particular purpose')
            settings.register_option(section, 'qemu_dst_bin',
                                     help_msg=help_msg,
                                     default=default_qemu_bin_path)

            help_msg = 'Accelerator used to run qemu (kvm or tcg)'
            settings.register_option(section, 'accel',
                                     help_msg=help_msg,
                                     default='kvm')

            help_msg = ('Whether to enable vhost for qemu (on/off/force). '
                        'Depends on nettype=bridge')
            settings.register_option(section, 'vhost',
                                     help_msg=help_msg,
                                     default='off')

            help_msg = 'Monitor type (human or qmp)'
            settings.register_option(section, 'monitor',
                                     help_msg=help_msg,
                                     default='')

            help_msg = 'Number of virtual cpus to use (1 or 2)'
            settings.register_option(section, 'smp',
                                     help_msg=help_msg,
                                     default='2')

            help_msg = 'Image format type to use (any valid qemu format)'
            settings.register_option(section, 'image_type',
                                     help_msg=help_msg,
                                     default=SUPPORTED_IMAGE_TYPES[0])

            help_msg = 'Guest network card model (any valid qemu card)'
            settings.register_option(section, 'nic_model',
                                     help_msg=help_msg,
                                     default=SUPPORTED_NIC_MODELS[0])

            help_msg = ('Guest disk bus for main image. One of ide, scsi, '
                        'virtio_blk, virtio_scsi, lsi_scsi, ahci, usb2 '
                        'or xenblk. Note: Older qemu versions and/or '
                        'operating systems (such as WinXP) might not support '
                        'virtio_scsi. Please use virtio_blk or ide instead.')
            settings.register_option(section, 'disk_bus',
                                     help_msg=help_msg,
                                     default=SUPPORTED_DISK_BUSES[0])

            help_msg = 'Enable qemu sandboxing (on/off)'
            settings.register_option(section, 'sandbox',
                                     help_msg=help_msg,
                                     default='on')

            help_msg = ('Prevent qemu from loading sysconfdir/qemu.conf '
                        'and sysconfdir/target-ARCH.conf at startup (yes/no)')
            settings.register_option(section, 'defconfig',
                                     help_msg=help_msg,
                                     default='yes')

            help_msg = ('Use MALLOC_PERTURB_ env variable set to 1 to help '
                        'catch memory allocation problems on qemu (yes/no)')
            settings.register_option(section, 'malloc_perturb',
                                     help_msg=help_msg,
                                     default='yes')

            # [vt.libvirt] section
            help_msg = ('Test connect URI for libvirt (qemu:///system, '
                        'lxc:///)')
            settings.register_option('vt.libvirt', 'connect_uri',
                                     help_msg=help_msg,
                                     default='qemu:///session')

            # [vt.debug] section
            help_msg = ('Do not clean up tmp files or VM processes at the end '
                        'of a virt-test execution')
            settings.register_option('vt.debug', 'no_cleanup',
                                     help_msg=help_msg, key_type=bool,
                                     default=False)

            # [plugins.vtjoblock] section
            help_msg = 'Directory in which to write the lock file'
            settings.register_option('plugins.vtjoblock', 'dir',
                                     help_msg=help_msg,
                                     default='/tmp')



import ast
import logging
import os.path
from six import StringIO

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser

from avocado.utils import distro


class ConfigError(Exception):

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class ConfigNoOptionError(ConfigError):

    def __init__(self, option, path):
        self.option = option
        self.path = path

    def __str__(self):
        return "There's no option %s in config file %s." % (
            self.option, self.path)


class LibvirtConfigUnknownKeyTypeError(ConfigError):

    def __init__(self, key, key_type):
        self.key = key
        self.key_type = key_type

    def __str__(self):
        return "Unknown type %s for key %s." % (self.key, self.key_type)


class LibvirtConfigUnknownKeyError(ConfigError):

    def __init__(self, key):
        self.key = key

    def __str__(self):
        return 'Unknown config key %s' % self.key


class SectionlessConfig(object):

    """
    This is a wrapper class for python's internal library configparser except
    allows manipulating sectionless configuration file with a dict-like way.

    Example config file test.conf:

    ># This is a comment line.
    >a = 1
    >b = [hi, there]
    >c = hello
    >d = "hi, there"
    >e = [hi,
    >    there]

    Example script using `try...finally...` statement:

    >>> from virttest import utils_config
    >>> config = utils_config.SectionlessConfig('test.conf')
    >>> try:
    ...     print len(config)
    ...     print config
    ...     print config['a']
    ...     del config['a']
    ...     config['f'] = 'test'
    ...     print config
    ... finally:
    ...     config.restore()

    Example script using `with` statement:

    >>> from virttest import utils_config
    >>> with utils_config.SectionlessConfig('test.conf') as config:
    ...     print len(config)
    ...     print config
    ...     print config['a']
    ...     del config['a']
    ...     config['f'] = 'test'
    ...     print config
    """

    def __init__(self, path):
        self.path = path
        self.parser = ConfigParser.ConfigParser()
        # Prevent of converting option names to lower case
        self.parser.optionxform = str
        self.backup_content = open(path, 'r').read()
        read_fp = StringIO('[root]\n' + self.backup_content)
        self.parser.readfp(read_fp)

    def __sync_file(self):
        out_file = open(self.path, 'w')
        try:
            out_file.write(self.__str__())
        finally:
            out_file.close()

    def __len__(self):
        return len(self.parser.items('root'))

    def __getitem__(self, option):
        try:
            return self.parser.get('root', option)
        except ConfigParser.NoOptionError:
            raise ConfigNoOptionError(option, self.path)

    def __setitem__(self, option, value):
        self.parser.set('root', option, value)
        self.__sync_file()

    def __delitem__(self, option):
        res = self.parser.remove_option('root', option)
        if res:
            self.__sync_file()
        else:
            raise ConfigNoOptionError(option, self.path)

    def __contains__(self, item):
        return self.parser.has_option('root', item)

    def __str__(self):
        write_fp = StringIO()
        self.parser.write(write_fp)
        return write_fp.getvalue().split('\n', 1)[1]

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.restore()

    def restore(self):
        out_file = open(self.path, 'w')
        try:
            out_file.write(self.backup_content)
        finally:
            out_file.close()

    def set_raw(self, option, value):
        self[option] = "%s" % value

    def set_string(self, option, value):
        self[option] = '"%s"' % value

    def set_int(self, option, value):
        self[option] = '%d' % int(value)

    def set_float(self, option, value):
        self[option] = '%s' % float(value)

    def set_boolean(self, option, value):
        if type(value) == str:
            value = int(value)
        if bool(value):
            self[option] = '1'
        else:
            self[option] = '0'

    def set_list(self, option, value):
        # TODO: line separation
        value = ['"%s"' % i for i in list(value)]
        self[option] = '[%s]' % ', '.join(value)

    def get_raw(self, option):
        return self[option]

    def get_string(self, option):
        raw_str = self[option].strip()
        if raw_str.startswith('"') and raw_str.endswith('"'):
            raw_str = raw_str[1:-1]
        elif raw_str.startswith("'") and raw_str.endswith("'"):
            raw_str = raw_str[1:-1]
        else:
            raise ValueError("Invalid value for string: %s" % raw_str)
        return raw_str

    def get_int(self, option):
        return int(self.get_raw(option))

    def get_float(self, option):
        return float(self.get_raw(option))

    def get_boolean(self, option):
        try:
            bool_str = self.get_string(option).lower()
        except ValueError:
            bool_str = str(self.get_int(option))

        if bool_str in ["1", "yes", "true", "on"]:
            return True
        if bool_str in ["0", "no", "false", "off"]:
            return False
        raise ValueError("Invalid value for boolean: %s" % bool_str)

    def get_list(self, option):
        list_str = self.get_raw(option)
        return [str(i) for i in ast.literal_eval(list_str)]


class LibvirtConfigCommon(SectionlessConfig):

    """
    A abstract class to manipulate options of a libvirt related configure files
    in a property's way.

    Variables "__option_types__" and "conf_path" must be setup in the
    inherented classes before use.

    "__option_types__" is a dict contains every possible option as keys and
    their type ("boolean", "int", "string", "float" or "list") as values.

    Basic usage:
    1) Create a config file object:
    >>> # LibvirtdConfig is a subclass of LibvirtConfigCommon.
    >>> config = LibvirtdConfig()

    2) Set or update an option:
    >>> config.listen_tcp = True
    >>> config.listen_tcp = 1
    >>> config.listen_tcp = "1" # All three have the same effect.

    >>> # If the setting value don't meet the specified type.
    >>> config.listen_tcp = "invalid"
    >>> # It'll thown an warning message and set a raw string instead.

    >>> # Use set_* methods when need to customize the result.
    >>> config.set_raw("'1'")

    3) Get an option:
    >>> is_listening = config.listen_tcp
    >>> print is_listening
    True

    4) Delete an option from the config file:
    >>> del config.listen_tcp

    5) Make the changes take effect in libvirt by restart libvirt daemon.
    >>> from virttest import utils_libvirtd
    >>> utils_libvirtd.Libvirtd().restart()

    6) Restore the content of the config file.
    >>> config.restore()
    """
    __option_types__ = {}
    conf_path = ''

    def __init__(self, path=''):
        if path:
            self.conf_path = path
        if not self.conf_path:
            raise ConfigError("Path for config file is not set up.")
        if not self.__option_types__:
            raise ConfigError("__option_types__ is not set up.")
        if not os.path.isfile(self.conf_path):
            raise ConfigError("Path for config file %s don't exists."
                              % self.conf_path)
        super(LibvirtConfigCommon, self).__init__(self.conf_path)

    def __getattr__(self, key):
        if key in self.__option_types__:
            key_type = self.__option_types__[key]
            if key_type not in ['boolean', 'int', 'float', 'string', 'list']:
                raise LibvirtConfigUnknownKeyTypeError(key, key_type)
            else:
                get_func = eval('self.get_' + key_type)
                try:
                    return get_func(key)
                except ConfigNoOptionError:
                    return None
        else:
            raise LibvirtConfigUnknownKeyError(key)

    def __setattr__(self, key, value):
        if key in self.__option_types__:
            key_type = self.__option_types__[key]
            if key_type not in ['boolean', 'int', 'float', 'string', 'list']:
                raise LibvirtConfigUnknownKeyTypeError(key, key_type)
            else:
                set_func = eval('self.set_' + key_type)
                try:
                    set_func(key, value)
                except ValueError:
                    logging.warning("Key %s might not have type %s. Set raw "
                                    "string instead.", key, key_type)
                    self.set_raw(key, value)
        super(LibvirtConfigCommon, self).__setattr__(key, value)

    def __delattr__(self, key):
        if key in self.__option_types__:
            key_type = self.__option_types__[key]
            if key_type not in ['boolean', 'int', 'float', 'string', 'list']:
                raise LibvirtConfigUnknownKeyTypeError(key, key_type)
            else:
                try:
                    del self[key]
                except ConfigNoOptionError:
                    pass
                super(LibvirtConfigCommon, self).__setattr__(key, None)
        else:
            raise LibvirtConfigUnknownKeyError(key)


class LibvirtConfig(LibvirtConfigCommon):

    """
    Class for libvirt config file.
    """
    conf_path = '/etc/libvirt/libvirt.conf'
    __option_types__ = {
        'uri_aliases': 'list',
        'uri_default': 'string',
        'remote_mode': 'string'
    }


class LibvirtdConfig(LibvirtConfigCommon):

    """
    Class for libvirt daemon config file.
    """
    conf_path = '/etc/libvirt/libvirtd.conf'
    __option_types__ = {
        'listen_tls': 'boolean',
        'listen_tcp': 'boolean',
        'tls_port': 'string',
        'tcp_port': 'string',
        'listen_addr': 'string',
        'mdns_adv': 'boolean',
        'mdns_name': 'string',
        'unix_sock_group': 'string',
        'unix_sock_ro_perms': 'string',
        'unix_sock_rw_perms': 'string',
        'unix_sock_dir': 'string',
        'auth_unix_ro': 'string',
        'auth_unix_rw': 'string',
        'auth_tcp': 'string',
        'auth_tls': 'string',
        'access_drivers': 'list',
        'key_file': 'string',
        'cert_file': 'string',
        'ca_file': 'string',
        'crl_file': 'string',
        'tls_no_sanity_certificate': 'boolean',
        'tls_no_verify_certificate': 'boolean',
        'tls_allowed_dn_list': 'list',
        'sasl_allowed_username_list': 'list',
        'max_clients': 'int',
        'max_queued_clients': 'int',
        'max_anonymous_clients': 'int',
        'min_workers': 'int',
        'max_workers': 'int',
        'prio_workers': 'int',
        'max_requests': 'int',
        'max_client_requests': 'int',
        'admin_min_workers': 'int',
        'admin_max_workers': 'int',
        'admin_max_clients': 'int',
        'admin_max_queued_clients': 'int',
        'admin_max_client_requests': 'int',
        'log_level': 'int',
        'log_filters': 'string',
        'log_outputs': 'string',
        'log_buffer_size': 'int',
        'audit_level': 'int',
        'audit_logging': 'int',
        'host_uuid': 'string',
        'host_uuid_source': 'string',
        'keepalive_interval': 'int',
        'keepalive_count': 'int',
        'keepalive_required': 'boolean',
        'admin_keepalive_required': 'int',
        'admin_keepalive_interval': 'int',
        'admin_keepalive_count': 'int',
        'unix_sock_admin_perms': 'string',
        'tls_priority': 'tls_priority',
        'ovs_timeout': 'int',
    }


class LibvirtQemuConfig(LibvirtConfigCommon):

    """
    Class for libvirt qemu config file.
    """
    conf_path = '/etc/libvirt/qemu.conf'
    __option_types__ = {
        'vnc_listen': 'string',
        'vnc_auto_unix_socket': 'boolean',
        'vnc_tls': 'boolean',
        'vnc_tls_x509_cert_dir': 'string',
        'vnc_tls_x509_verify': 'boolean',
        'vnc_password': 'string',
        'vnc_sasl': 'boolean',
        'vnc_sasl_dir': 'string',
        'vnc_allow_host_audio': 'boolean',
        'spice_listen': 'string',
        'spice_tls': 'boolean',
        'spice_tls_x509_cert_dir': 'string',
        'spice_password': 'string',
        'remote_display_port_min': 'int',
        'remote_display_port_max': 'int',
        'remote_websocket_port_min': 'int',
        'remote_websocket_port_max': 'int',
        'security_driver': 'list',
        'security_default_confined': 'boolean',
        'security_require_confined': 'boolean',
        'user': 'string',
        'group': 'string',
        'dynamic_ownership': 'boolean',
        'cgroup_controllers': 'list',
        'cgroup_device_acl': 'list',
        'save_image_format': 'string',
        'dump_image_format': 'string',
        'snapshot_image_format': 'string',
        'auto_dump_path': 'string',
        'auto_dump_bypass_cache': 'boolean',
        'auto_start_bypass_cache': 'boolean',
        'hugetlbfs_mount': 'list',
        'bridge_helper': 'string',
        'clear_emulator_capabilities': 'boolean',
        'set_process_name': 'boolean',
        'max_processes': 'int',
        'max_files': 'int',
        'mac_filter': 'boolean',
        'relaxed_acs_check': 'boolean',
        'allow_disk_format_probing': 'boolean',
        'lock_manager': 'string',
        'max_queued': 'int',
        'keepalive_interval': 'int',
        'keepalive_count': 'int',
        'seccomp_sandbox': 'int',
        'migration_address': 'string',
        'migration_port_min': 'int',
        'migration_port_max': 'int',
        'namespaces': 'list',
        'vnc_tls_x509_secret_uuid': 'string',
        'default_tls_x509_cert_dir': 'string',
        'default_tls_x509_verify': 'boolean',
        'default_tls_x509_secret_uuid': 'string',
        'spice_auto_unix_socket': 'boolean',
        'spice_sasl': 'boolean',
        'spice_sasl_dir': 'string',
        'chardev_tls': 'boolean',
        'chardev_tls_x509_cert_dir': 'string',
        'chardev_tls_x509_verify': 'boolean',
        'chardev_tls_x509_secret_uuid': 'string',
        'migrate_tls_x509_cert_dir': 'string',
        'migrate_tls_x509_verify': 'boolean',
        'migrate_tls_x509_secret_uuid': 'string',
        'nographics_allow_host_audio': 'boolean',
        'remember_owner': 'boolean',
        'pr_helper': 'string',
        'slirp_helper': 'string',
        'max_core': 'string',
        'dump_guest_core': 'boolean',
        'stdio_handler': 'string',
        'max_threads_per_process': 'int',
        'migration_host': 'string',
        'log_timestamp': 'boolean',
        'nvram': 'list',
        'gluster_debug_level': 'int',
        'memory_backing_dir': 'string',
        'vxhs_tls': 'boolean',
        'vxhs_tls_x509_cert_dir': 'string',
        'nbd_tls': 'boolean',
        'nbd_tls_x509_cert_dir': 'string',
        'swtpm_user': 'string',
        'swtpm_group': 'string',
        'capability_filters': 'list',
        'nbd_tls_x509_secret_uuid': 'string',
        'backup_tls_x509_cert_dir': 'string',
        'backup_tls_x509_verify': 'boolean',
        'backup_tls_x509_secret_uuid': 'string',
    }


class LibvirtdSysConfig(LibvirtConfigCommon):

    """
    Class for sysconfig libvirtd config file.
    """
    if distro.detect().name == 'Ubuntu':
        conf_path = '/etc/default/libvirtd'
    else:
        conf_path = '/etc/sysconfig/libvirtd'
    __option_types__ = {
        'LIBVIRTD_CONFIG': 'string',
        'LIBVIRTD_ARGS': 'string',
        'KRB5_KTNAME': 'string',
        'QEMU_AUDIO_DRV': 'string',
        'SDL_AUDIODRIVER': 'string',
        'LIBVIRTD_NOFILES_LIMIT': 'int',
        'DAEMON_COREFILE_LIMIT': 'string',
    }


class LibvirtGuestsConfig(LibvirtConfigCommon):

    """
    Class for sysconfig libvirt-guests config file.
    """
    conf_path = '/etc/sysconfig/libvirt-guests'
    __option_types__ = {
        'URIS': 'string',
        'ON_BOOT': 'string',
        'START_DELAY': 'int',
        'ON_SHUTDOWN': 'string',
        'PARALLEL_SHUTDOWN': 'int',
        'SHUTDOWN_TIMEOUT': 'int',
        'BYPASS_CACHE': 'boolean',
        'SYNC_TIME': 'boolean',
    }


class LibvirtSanLockConfig(LibvirtConfigCommon):

    """
    Class for libvirt san lock config file.
    """
    conf_path = '/etc/libvirt/qemu-sanlock.conf'
    __option_types__ = {
        'auto_disk_leases': 'boolean',
        'disk_lease_dir': 'string',
        'host_id': 'int',
        'require_lease_for_disks': 'boolean',
        'user': 'string',
        'group': 'string',
    }


class VirtQemudConfig(LibvirtConfigCommon):

    """
    Class for libvirt virtqemud config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtqemud.conf'
    # The virtqemud attribue values are quite the same with previous libvirtd.conf except removal of tls,tcp
    # certificate part group
    __option_types__ = {
        'mdns_adv': 'boolean',
        'mdns_name': 'string',

        'unix_sock_group': 'string',
        'unix_sock_ro_perms': 'string',
        'unix_sock_rw_perms': 'string',
        'unix_sock_admin_perms': 'string',
        'unix_sock_dir': 'string',

        'auth_unix_ro': 'string',
        'auth_unix_rw': 'string',
        'access_drivers': 'list',

        'sasl_allowed_username_list': 'list',

        'max_clients': 'int',
        'max_queued_clients': 'int',
        'max_anonymous_clients': 'int',
        'min_workers': 'int',
        'max_workers': 'int',
        'prio_workers': 'int',
        'max_client_requests': 'int',
        'admin_min_workers': 'int',
        'admin_max_workers': 'int',
        'admin_max_clients': 'int',
        'admin_max_queued_clients': 'int',
        'admin_max_client_requests': 'int',

        'log_level': 'int',
        'log_filters': 'string',
        'log_outputs': 'string',

        'audit_level': 'int',
        'audit_logging': 'int',
        'host_uuid': 'string',
        'host_uuid_source': 'string',

        'keepalive_interval': 'int',
        'keepalive_count': 'int',
        'keepalive_required': 'boolean',
        'admin_keepalive_required': 'int',

        'ovs_timeout': 'int',
    }


class VirtInterfacedConfig(VirtQemudConfig):

    """
    Class for libvirt virtinterfaced config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtinterfaced.conf'


class VirtLockdConfig(VirtQemudConfig):

    """
    Class for libvirt virtlock config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtlockd.conf'


class VirtLogdConfig(VirtQemudConfig):

    """
    Class for libvirt virtlogd config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtlogd.conf'


class VirtNetworkdConfig(VirtQemudConfig):

    """
    Class for libvirt virtnetworkd config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtnetworkd.conf'


class VirtNodedevdConfig(VirtQemudConfig):

    """
    Class for libvirt virtnodedevd config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtnodedevd.conf'


class VirtNwfilterdConfig(VirtQemudConfig):

    """
    Class for libvirt virtnwfilterd config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtnwfilterd.conf'


class VirtSecretdConfig(VirtQemudConfig):

    """
    Class for libvirt virtsecretd config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtsecretd.conf'


class VirtStoragedConfig(VirtQemudConfig):

    """
    Class for libvirt virtstoraged config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtstoraged.conf'


class VirtProxydConfig(VirtQemudConfig):

    """
    Class for libvirt virtproxyd config file.
    This is used to represent split daemons changes after libvirt version 5.6.0
    """
    conf_path = '/etc/libvirt/virtproxyd.conf'
    # The virtproxyd.conf attribue values are quite the same with VirtQemud config with additional tls,tcp
    # certificate part group
    __option_types__ = {
        'listen_tls': 'boolean',
        'listen_tcp': 'boolean',
        'tls_port': 'string',
        'tcp_port': 'string',
        'listen_addr': 'string',

        'mdns_adv': 'boolean',
        'mdns_name': 'string',

        'unix_sock_group': 'string',
        'unix_sock_ro_perms': 'string',
        'unix_sock_rw_perms': 'string',
        'unix_sock_admin_perms': 'string',
        'unix_sock_dir': 'string',

        'auth_unix_ro': 'string',
        'auth_unix_rw': 'string',
        'access_drivers': 'list',

        'sasl_allowed_username_list': 'list',

        'max_clients': 'int',
        'max_queued_clients': 'int',
        'max_anonymous_clients': 'int',
        'min_workers': 'int',
        'max_workers': 'int',
        'prio_workers': 'int',
        'max_client_requests': 'int',
        'admin_min_workers': 'int',
        'admin_max_workers': 'int',
        'admin_max_clients': 'int',
        'admin_max_queued_clients': 'int',
        'admin_max_client_requests': 'int',

        'log_level': 'int',
        'log_filters': 'string',
        'log_outputs': 'string',

        'audit_level': 'int',
        'audit_logging': 'int',
        'host_uuid': 'string',
        'host_uuid_source': 'string',

        'keepalive_interval': 'int',
        'keepalive_count': 'int',
        'keepalive_required': 'boolean',
        'admin_keepalive_required': 'int',

        'ovs_timeout': 'int',
    }


def get_conf_obj(config_type):
    """
    Wrapper method to get the object from the given configuration type

    :param config_type: The configuration type to get
        For example, "libvirtd" or "virtqemud"
    :return: utils_config.LibvirtConfigCommon object
    """
    return {
        'libvirt': LibvirtConfig,
        'libvirtd': LibvirtdConfig,
        'qemu': LibvirtQemuConfig,
        'sysconfig': LibvirtdSysConfig,
        'guestconfig': LibvirtGuestsConfig,
        'virtqemud': VirtQemudConfig,
        'virtproxyd': VirtProxydConfig,
        'virtnetworkd': VirtNetworkdConfig,
        'virtstoraged': VirtStoragedConfig,
        'virtsecretd': VirtSecretdConfig,
        'virtinterfaced': VirtInterfacedConfig,
        'virtnodedevd': VirtNodedevdConfig,
        'virtnwfilterd': VirtNwfilterdConfig
         }.get(config_type)()
